/*
 * Class created by lensfrex.
 */

package net.lensfrex.dscape.web.service.user.admin;

import cn.dev33.satoken.stp.StpUtil;
import cn.dev33.satoken.util.SaFoxUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import net.lensfrex.dscape.dao.entity.BlackList;
import net.lensfrex.dscape.dao.entity.UserBasic;
import net.lensfrex.dscape.dao.service.BlackListService;
import net.lensfrex.dscape.dao.service.UserBasicService;
import net.lensfrex.dscape.dto.response.general.ResponseCode;
import net.lensfrex.dscape.enums.user.BlackListTypeEnum;
import net.lensfrex.dscape.exception.GlobalException;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service
public class AdminService {
    private final RedisTemplate<String, String> redis;

    private final UserBasicService userBasicService;

    private final BlackListService blackListService;

    public AdminService(RedisTemplate<String, String> redis,
                        UserBasicService userBasicService,
                        BlackListService blackListService) {

        this.redis = redis;
        this.userBasicService = userBasicService;
        this.blackListService = blackListService;
    }

    private static final int INVITE_CODE_LENGTH = 6;

    private static final String REGISTER_INVITE_CODE_KEY = "dscape:user:inviteCodes";

    private static final String ADMIN_INVITE_CODE_COUNTER_KEY = "dscape:admin:inviteCodeCounter:%s";

    public List<String> generateInviteCode(long expired, int count) {
        String adminUid = StpUtil.getLoginIdAsString();
        String adminInviteCodeCounterKey = String.format(ADMIN_INVITE_CODE_COUNTER_KEY, adminUid);

        String countResult = redis.opsForValue().get(adminInviteCodeCounterKey);

        if (countResult != null && Integer.parseInt(countResult) > 1024) {
            throw new GlobalException(ResponseCode.INVITE_CODE_REACHED_LIMIT);
        }

        List<String> inviteCodes = new ArrayList<>(count);

        for (int i = 0; i < count; i++) {
            inviteCodes.add(SaFoxUtil.getRandomString(INVITE_CODE_LENGTH));
        }
        HashMap<String, String> codeAdmin = new HashMap<>(count);
        inviteCodes.forEach(inviteCode -> codeAdmin.put(inviteCode, adminUid));

        redis.opsForHash().putAll(REGISTER_INVITE_CODE_KEY, codeAdmin);
        redis.opsForValue().increment(adminInviteCodeCounterKey, count);

        return inviteCodes;
    }

    private static final String USER_INFO_CACHE_KEY = "dscape:user:basic";

    public void modifyUserStatus(String username, int status) {
        UpdateWrapper<UserBasic> wrapper = new UpdateWrapper<>();
        wrapper.lambda()
                .eq(UserBasic::getUserName, username)
                .eq(UserBasic::getSuperior, StpUtil.getLoginIdAsString())
                .set(UserBasic::getStatus, status);

        boolean success = userBasicService.update(wrapper);

        if (success) {
            this.clearUserBasicCache(username);
        }
    }

    public void addUserToBlackList(String username) {
        BlackList blackList = new BlackList();
        blackList.setData(username);
        blackList.setType(BlackListTypeEnum.UID.getCode());

        boolean success = blackListService.save(blackList);
    }

    public void deleteUser(String username) {
        UpdateWrapper<UserBasic> wrapper = new UpdateWrapper<>();
        wrapper.lambda()
                .eq(UserBasic::getUserName, username)
                .set(UserBasic::getIsDeleted, 1);

        boolean success = userBasicService.update(wrapper);

        if (success) {
            this.clearUserBasicCache(username);
        }
    }

    private void clearUserBasicCache(String username) {
        redis.opsForHash().delete(USER_INFO_CACHE_KEY, username);
    }
}